@@ -130,27 +130,32 @@ sub setup ($) {
 	    VARNAME => 'chroot',
 	    GROUP => 'Chroot options',
 	    DEFAULT => undef,
-	    HELP => 'Default chroot (defaults to distribution[-arch][-sbuild])'
+	    HELP => 'Default chroot (defaults to distribution[-arch][-sbuild])',
+	    CLI_OPTIONS => ['-c', '--chroot']
 	},
 	'BUILD_ARCH_ALL'			=> {
 	    TYPE => 'BOOL',
 	    VARNAME => 'build_arch_all',
 	    GROUP => 'Build options',
 	    DEFAULT => 0,
-	    HELP => 'Build architecture: all packages by default'
+	    HELP => 'Build architecture: all packages by default.',
+	    CLI_OPTIONS => ['--arch-all', '--no-arch-all']
 	},
 	'BUILD_ARCH_ANY'			=> {
 	    TYPE => 'BOOL',
 	    VARNAME => 'build_arch_any',
 	    GROUP => 'Build options',
 	    DEFAULT => 1,
-	    HELP => 'Build architecture: any packages by default'
+	    HELP => 'Build architecture: any packages by default.',
+	    CLI_OPTIONS => ['--arch-any', '--no-arch-any']
 	},
 	'NOLOG'					=> {
 	    TYPE => 'BOOL',
-	    GROUP => '__INTERNAL',
+	    VARNAME => 'nolog',
+	    GROUP => 'Logging options',
 	    DEFAULT => 0,
-	    HELP => 'Disable use of log file'
+	    HELP => 'Disable use of log file',
+	    CLI_OPTIONS => ['-n', '--nolog']
 	},
 	'SUDO'					=> {
 	    TYPE => 'STRING',
@@ -220,6 +225,68 @@ sub setup ($) {
 	    DEFAULT => ['-q'],
 	    HELP => 'Additional command-line options for schroot'
 	},
+	'AUTOPKGTEST_VIRT_SERVER'			=> {
+	    TYPE => 'STRING',
+	    VARNAME => 'autopkgtest_virt_server',
+	    GROUP => 'Programs',
+	    CHECK => sub {
+		my $conf = shift;
+		my $entry = shift;
+		my $key = $entry->{'NAME'};
+		my $program = $conf->get($key);
+
+		# if the autopkgtest virtualization server name is only letters
+		# a-z then it is missing the autopkgtest-virt- prefix
+		if ($program =~ /^[a-z]+$/) {
+		    $conf->set($key, "autopkgtest-virt-$program");
+		}
+
+		# Only validate if needed.
+		if ($conf->get('CHROOT_MODE') eq 'autopkgtest') {
+		    $validate_program->($conf, $entry);
+		}
+	    },
+	    DEFAULT => 'autopkgtest-virt-schroot',
+	    HELP => 'Path to autopkgtest-virt-* binary, selecting the virtualization server.',
+	    CLI_OPTIONS => ['--autopkgtest-virt-server']
+	},
+	'AUTOPKGTEST_VIRT_SERVER_OPTIONS'			=> {
+	    TYPE => 'ARRAY:STRING',
+	    VARNAME => 'autopkgtest_virt_server_options',
+	    GROUP => 'Programs',
+	    DEFAULT => [],
+	    GET => sub {
+		my $conf = shift;
+		my $entry = shift;
+
+		my $retval = $conf->_get($entry->{'NAME'});
+
+		my $dist = $conf->get('DISTRIBUTION');
+		my $hostarch = $conf->get('HOST_ARCH');
+		my %percent = (
+		    '%' => '%',
+		    'a' => $hostarch, 'SBUILD_HOST_ARCH' => $hostarch,
+		    'r' => $dist, 'SBUILD_DISTRIBUTION' => $dist,
+		);
+
+		my $keyword_pat = join("|",
+		    sort {length $b <=> length $a || $a cmp $b} keys %percent);
+		foreach (@{$retval}) {
+		    s{
+			# Match a percent followed by a valid keyword
+			\%($keyword_pat)
+		    }{
+			# Substitute with the appropriate value only if it's defined
+			$percent{$1} || $&
+		    }msxge;
+		}
+		return $retval;
+	    },
+	    HELP => 'Additional command-line options for autopkgtest-virt-*',
+	    CLI_OPTIONS => ['--autopkgtest-virt-server-opt', '--autopkgtest-virt-server-opts']
+	},
+	# Do not check for the existance of fakeroot because it's needed
+	# inside the chroot and not on the host
 	'FAKEROOT'				=> {
 	    TYPE => 'STRING',
 	    VARNAME => 'fakeroot',
@@ -278,9 +345,11 @@ sub setup ($) {
 	},
 	'DPKG_BUILDPACKAGE_USER_OPTIONS'	=> {
 	    TYPE => 'ARRAY:STRING',
-	    GROUP => '__INTERNAL',
+	    VARNAME => 'dpkg_buildpackage_user_options',
+	    GROUP => 'Programs',
 	    DEFAULT => [],
-	    HELP => 'Additional command-line options for dpkg-buildpackage.  Not settable in config.'
+	    HELP => 'Additional command-line options for dpkg-buildpackage.',
+	    CLI_OPTIONS => ['--debbuildopt', '--debbuildopts', '--jobs']
 	},
 	'DPKG_SOURCE'				=> {
 	    TYPE => 'STRING',
@@ -295,7 +364,8 @@ sub setup ($) {
 	    VARNAME => 'dpkg_source_opts',
 	    GROUP => 'Programs',
 	    DEFAULT => [],
-	    HELP => 'Additional command-line options for dpkg-source'
+	    HELP => 'Additional command-line options for dpkg-source',
+	    CLI_OPTIONS => ['--dpkg-source-opt', '--dpkg-source-opts']
 	},
 	'DCMD'					=> {
 	    TYPE => 'STRING',
@@ -317,8 +387,10 @@ sub setup ($) {
 	    TYPE => 'STRING',
 	    VARNAME => 'stats_dir',
 	    GROUP => 'Statistics',
+	    IGNORE_DEFAULT => 1, # Don't dump the current home
 	    DEFAULT => "$HOME/stats",
-	    HELP => 'Directory for writing build statistics to'
+	    HELP => 'Directory for writing build statistics to',
+	    CLI_OPTIONS => ['--stats-dir']
 	},
 	'PACKAGE_CHECKLIST'			=> {
 	    TYPE => 'STRING',
@@ -332,7 +404,7 @@ sub setup ($) {
 	    VARNAME => 'build_env_cmnd',
 	    GROUP => 'Build options',
 	    DEFAULT => "",
-	    HELP => 'This command is run with the dpkg-buildpackage command line passed to it (in the chroot, if doing a chrooted build).  It is used by the sparc buildd (which is sparc64) to call the wrapper script that sets the environment to sparc (32-bit).  It could be used for other build environment setup scripts.  Note that this is superceded by schroot\'s \'command-prefix\' option'
+	    HELP => 'This command is run with the dpkg-buildpackage command line passed to it (in the chroot, if doing a chrooted build).  It is used by the sparc buildd (which is sparc64) to call the wrapper script that sets the environment to sparc (32-bit).  It could be used for other build environment setup scripts.  Note that this is superseded by schroot\'s \'command-prefix\' option'
 	},
 	'PGP_OPTIONS'				=> {
 	    TYPE => 'ARRAY:STRING',
@@ -448,7 +520,8 @@ sub setup ($) {
 		return $retval;
 	    },
 	    DEFAULT => "",
-	    HELP => 'email address to mail build logs to'
+	    HELP => 'email address to mail build logs to',
+	    CLI_OPTIONS => ['--mail-log-to']
 	},
 	'MAILTO_FORCED_BY_CLI'			=> {
 	    TYPE => 'BOOL',
@@ -467,7 +540,8 @@ sub setup ($) {
 	    VARNAME => 'mailfrom',
 	    GROUP => 'Logging options',
 	    DEFAULT => "Source Builder <sbuild>",
-	    HELP => 'email address set in the From line of build logs'
+	    HELP => 'email address set in the From line of build logs',
+	    CLI_OPTIONS => ['--mailfrom']
 	},
 	'COMPRESS_BUILD_LOG_MAILS'              => {
 	    TYPE => 'BOOL',
@@ -498,7 +572,8 @@ sub setup ($) {
 			     qw(always successful never));
 	    },
 	    DEFAULT => 'always',
-	    HELP => 'When to purge the build dependencies after a build; possible values are "never", "successful", and "always"'
+	    HELP => 'When to purge the build dependencies after a build; possible values are "never", "successful", and "always"',
+	    CLI_OPTIONS => ['-p', '--purge', '--purge-deps']
 	},
 	'PURGE_BUILD_DIRECTORY'			=> {
 	    TYPE => 'STRING',
@@ -515,7 +590,8 @@ sub setup ($) {
 			     qw(always successful never));
 	    },
 	    DEFAULT => 'always',
-	    HELP => 'When to purge the build directory after a build; possible values are "never", "successful", and "always"'
+	    HELP => 'When to purge the build directory after a build; possible values are "never", "successful", and "always"',
+	    CLI_OPTIONS => ['-p', '--purge', '--purge-build']
 	},
 	'PURGE_SESSION'			=> {
 	    TYPE => 'STRING',
@@ -532,7 +608,8 @@ sub setup ($) {
 			     qw(always successful never));
 	    },
 	    DEFAULT => 'always',
-	    HELP => 'Purge the schroot session following a build.  This is useful in conjunction with the --purge and --purge-deps options when using snapshot chroots, since by default the snapshot will be deleted. Possible values are "always" (default), "never", and "successful"'
+	    HELP => 'Purge the schroot session following a build.  This is useful in conjunction with the --purge and --purge-deps options when using snapshot chroots, since by default the snapshot will be deleted. Possible values are "always" (default), "never", and "successful"',
+	    CLI_OPTIONS => ['-p', '--purge', '--purge-session']
 	},
 	'TOOLCHAIN_REGEX'			=> {
 	    TYPE => 'ARRAY:STRING',
@@ -571,7 +648,7 @@ sub setup ($) {
 	    VARNAME => 'lock_interval',
 	    GROUP => 'Build timeouts',
 	    DEFAULT => 5,
-	    HELP => 'Lock wait interval (seconds).  Maximum wait time is (max_lock_trys × lock_interval).'
+	    HELP => 'Lock wait interval (seconds).  Maximum wait time is (max_lock_trys x lock_interval).'
 	},
 	'CHROOT_MODE'				=> {
 	    TYPE => 'STRING',
@@ -584,10 +661,11 @@ sub setup ($) {
 
 		die "Bad chroot mode \'" . $conf->get('CHROOT_MODE') . "\'"
 		    if !isin($conf->get('CHROOT_MODE'),
-			     qw(schroot sudo));
+			     qw(schroot sudo autopkgtest));
 	    },
 	    DEFAULT => 'schroot',
-	    HELP => 'Mechanism to use for chroot virtualisation.  Possible value are "schroot" (default) and "sudo".'
+	    HELP => 'Mechanism to use for chroot virtualisation.  Possible value are "schroot" (default), "sudo" and "autopkgtest".',
+	    CLI_OPTIONS => ['--chroot-mode']
 	},
 	'CHROOT_SPLIT'				=> {
 	    TYPE => 'BOOL',
@@ -613,26 +691,37 @@ sub setup ($) {
 	    CHECK => $validate_directory,
 	    HELP => 'This option is deprecated.  Directory for chroot symlinks and sbuild logs.  Defaults to the current directory if unspecified.  It is used as the location of chroot symlinks (obsolete) and for current build log symlinks and some build logs.  There is no default; if unset, it defaults to the current working directory.  $HOME/build is another common configuration.'
 	},
+	'BUILD_PATH'				=> {
+	    TYPE => 'STRING',
+	    VARNAME => 'build_path',
+	    GROUP => 'Build options',
+	    DEFAULT => undef,
+	    HELP => 'By default the package is built in a path of the following format /build/packagename-XXXXXX/packagename-version/ where XXXXXX is a random ascii string. This option allows one to specify a custom path where the package is built inside the chroot. Notice that the sbuild user in the chroot must have permissions to create the path. Common writable locations are subdirectories of /tmp or /build. The buildpath must be an empty directory because the last component of the path will be removed after the build is finished. If you are running multiple sbuild instances with the same build path in parallel for the same package, make sure that your build path is not in a directory commonly mounted by all sbuild instances (like /tmp or /home). In that case, use for example /build instead. Otherwise, your builds will probably fail or contain wrong content.',
+	    CLI_OPTIONS => ['--build-path']
+	},
 	'SBUILD_MODE'				=> {
 	    TYPE => 'STRING',
 	    VARNAME => 'sbuild_mode',
 	    GROUP => 'Core options',
 	    DEFAULT => 'user',
-	    HELP => 'sbuild behaviour; possible values are "user" (exit status reports build failures) and "buildd" (exit status does not report build failures) for use in a buildd setup.  "buildd" also currently implies enabling of "legacy features" such as chroot symlinks in the build directory and the creation of current symlinks in the build directory.'
+	    HELP => 'sbuild behaviour; possible values are "user" (exit status reports build failures) and "buildd" (exit status does not report build failures) for use in a buildd setup.  "buildd" also currently implies enabling of "legacy features" such as chroot symlinks in the build directory and the creation of current symlinks in the build directory.',
+	    CLI_OPTIONS => ['--sbuild-mode']
 	},
 	'CHROOT_SETUP_SCRIPT'				=> {
 	    TYPE => 'STRING',
 	    VARNAME => 'chroot_setup_script',
 	    GROUP => 'Chroot options',
 	    DEFAULT => undef,
-	    HELP => 'Script to run to perform custom setup tasks in the chroot.'
+	    HELP => 'Script to run to perform custom setup tasks in the chroot.',
+	    CLI_OPTIONS => ['--setup-hook']
 	},
 	'FORCE_ORIG_SOURCE'			=> {
 	    TYPE => 'BOOL',
 	    VARNAME => 'force_orig_source',
 	    GROUP => 'Build options',
 	    DEFAULT => 0,
-	    HELP => 'By default, the -s option only includes the .orig.tar.gz when needed (i.e. when the Debian revision is 0 or 1).  By setting this option to 1, the .orig.tar.gz will always be included when -s is used.  This is equivalent to --force-orig-source.'
+	    HELP => 'By default, the -s option only includes the .orig.tar.gz when needed (i.e. when the Debian revision is 0 or 1).  By setting this option to 1, the .orig.tar.gz will always be included when -s is used.',
+	    CLI_OPTIONS => ['--force-orig-source']
 	},
 	'INDIVIDUAL_STALLED_PKG_TIMEOUT'	=> {
 	    TYPE => 'HASH:NUMERIC',
@@ -678,7 +767,8 @@ sub setup ($) {
 	    VARNAME => 'ld_library_path',
 	    GROUP => 'Build environment',
 	    DEFAULT => undef,
-	    HELP => 'Library search path to use inside the chroot.'
+	    HELP => 'Library search path to use inside the chroot.',
+	    CLI_OPTIONS => ['--use-snapshot']
 	},
 	'MAINTAINER_NAME'			=> {
 	    TYPE => 'STRING',
@@ -686,7 +776,8 @@ sub setup ($) {
 	    GROUP => 'Maintainer options',
 	    DEFAULT => undef,
 	    SET => $set_signing_option,
-	    HELP => 'Name to use as override in .changes files for the Maintainer field.  The Maintainer field will not be overridden unless set here.'
+	    HELP => 'Name to use as override in .changes files for the Maintainer field.  The Maintainer field will not be overridden unless set here.',
+	    CLI_OPTIONS => ['-m', '--maintainer']
 	},
 	'UPLOADER_NAME'				=> {
 	    VARNAME => 'uploader_name',
@@ -694,14 +785,16 @@ sub setup ($) {
 	    GROUP => 'Maintainer options',
 	    DEFAULT => undef,
 	    SET => $set_signing_option,
-	    HELP => 'Name to use as override in .changes file for the Changed-By: field.'
+	    HELP => 'Name to use as override in .changes file for the Changed-By: field.',
+	    CLI_OPTIONS => ['-e', '--uploader']
 	},
 	'KEY_ID'				=> {
 	    TYPE => 'STRING',
 	    VARNAME => 'key_id',
 	    GROUP => 'Maintainer options',
 	    DEFAULT => undef,
-	    HELP => 'Key ID to use in .changes for the current upload.  It overrides both $maintainer_name and $uploader_name.'
+	    HELP => 'Key ID to use in .changes for the current upload.  It overrides both $maintainer_name and $uploader_name.',
+	    CLI_OPTIONS => ['-k', '--keyid']
 	},
 	'SIGNING_OPTIONS'			=> {
 	    TYPE => 'STRING',
@@ -714,14 +807,16 @@ sub setup ($) {
 	    VARNAME => 'apt_clean',
 	    GROUP => 'Chroot options',
 	    DEFAULT => 0,
-	    HELP => 'APT clean.  1 to enable running "apt-get clean" at the start of each build, or 0 to disable.'
+	    HELP => 'APT clean.  1 to enable running "apt-get clean" at the start of each build, or 0 to disable.',
+	    CLI_OPTIONS => ['--apt-clean', '--no-apt-clean']
 	},
 	'APT_UPDATE'				=> {
 	    TYPE => 'BOOL',
 	    VARNAME => 'apt_update',
 	    GROUP => 'Chroot options',
 	    DEFAULT => 1,
-	    HELP => 'APT update.  1 to enable running "apt-get update" at the start of each build, or 0 to disable.'
+	    HELP => 'APT update.  1 to enable running "apt-get update" at the start of each build, or 0 to disable.',
+	    CLI_OPTIONS => ['--apt-update', '--no-apt-update']
 	},
 	'APT_UPDATE_ARCHIVE_ONLY'		=> {
 	    TYPE => 'BOOL',
@@ -735,14 +830,16 @@ sub setup ($) {
 	    VARNAME => 'apt_upgrade',
 	    GROUP => 'Chroot options',
 	    DEFAULT => 0,
-	    HELP => 'APT upgrade.  1 to enable running "apt-get upgrade" at the start of each build, or 0 to disable.'
+	    HELP => 'APT upgrade.  1 to enable running "apt-get upgrade" at the start of each build, or 0 to disable.',
+	    CLI_OPTIONS => ['--apt-upgrade', '--no-apt-upgrade']
 	},
 	'APT_DISTUPGRADE'			=> {
 	    TYPE => 'BOOL',
 	    VARNAME => 'apt_distupgrade',
 	    GROUP => 'Chroot options',
 	    DEFAULT => 1,
-	    HELP => 'APT distupgrade.  1 to enable running "apt-get dist-upgrade" at the start of each build, or 0 to disable.'
+	    HELP => 'APT distupgrade.  1 to enable running "apt-get dist-upgrade" at the start of each build, or 0 to disable.',
+	    CLI_OPTIONS => ['--apt-distupgrade', '--no-apt-distupgrade']
 	},
 	'APT_ALLOW_UNAUTHENTICATED'		=> {
 	    TYPE => 'BOOL',
@@ -753,94 +850,112 @@ sub setup ($) {
 	},
 	'BATCH_MODE'				=> {
 	    TYPE => 'BOOL',
-	    GROUP => '__INTERNAL',
+	    VARNAME => 'batch_mode',
+	    GROUP => 'Core options',
 	    DEFAULT => 0,
-	    HELP => 'Enable batch mode?'
+	    HELP => 'Enable batch mode?',
+	    CLI_OPTIONS => ['-b', '--batch']
 	},
 	'CORE_DEPENDS'				=> {
 	    TYPE => 'ARRAY:STRING',
 	    VARNAME => 'core_depends',
 	    GROUP => 'Core options',
-	    DEFAULT => ['build-essential', 'fakeroot'],
+	    DEFAULT => ['build-essential:native', 'fakeroot:native'],
 	    HELP => 'Packages which must be installed in the chroot for all builds.'
 	},
 	'MANUAL_DEPENDS'			=> {
 	    TYPE => 'ARRAY:STRING',
-	    GROUP => '__INTERNAL',
+	    VARNAME => 'manual_depends',
+	    GROUP => 'Core options',
 	    DEFAULT => [],
-	    HELP => 'Additional per-build dependencies.  Do not set by hand.'
+	    HELP => 'Additional per-build dependencies.',
+	    CLI_OPTIONS => ['--add-depends']
 	},
 	'MANUAL_CONFLICTS'			=> {
 	    TYPE => 'ARRAY:STRING',
-	    GROUP => '__INTERNAL',
+	    VARNAME => 'manual_conflicts',
+	    GROUP => 'Core options',
 	    DEFAULT => [],
-	    HELP => 'Additional per-build dependencies.  Do not set by hand.'
+	    HELP => 'Additional per-build dependencies.',
+	    CLI_OPTIONS => ['--add-conflicts']
 	},
 	'MANUAL_DEPENDS_ARCH'			=> {
 	    TYPE => 'ARRAY:STRING',
-	    GROUP => '__INTERNAL',
+	    VARNAME => 'manual_depends_arch',
+	    GROUP => 'Core options',
 	    DEFAULT => [],
-	    HELP => 'Additional per-build dependencies.  Do not set by hand.'
+	    HELP => 'Additional per-build dependencies.',
+	    CLI_OPTIONS => ['--add-depends-arch']
 	},
 	'MANUAL_CONFLICTS_ARCH'			=> {
 	    TYPE => 'ARRAY:STRING',
-	    GROUP => '__INTERNAL',
+	    VARNAME => 'manual_conflicts_arch',
+	    GROUP => 'Core options',
 	    DEFAULT => [],
-	    HELP => 'Additional per-build dependencies.  Do not set by hand.'
+	    HELP => 'Additional per-build dependencies.',
+	    CLI_OPTIONS => ['--add-conflicts-arch']
 	},
 	'MANUAL_DEPENDS_INDEP'			=> {
 	    TYPE => 'ARRAY:STRING',
-	    GROUP => '__INTERNAL',
+	    VARNAME => 'manual_depends_indep',
+	    GROUP => 'Core options',
 	    DEFAULT => [],
-	    HELP => 'Additional per-build dependencies.  Do not set by hand.'
+	    HELP => 'Additional per-build dependencies.',
+	    CLI_OPTIONS => ['--add-depends-indep']
 	},
 	'MANUAL_CONFLICTS_INDEP'		=> {
 	    TYPE => 'ARRAY:STRING',
-	    GROUP => '__INTERNAL',
+	    VARNAME => 'manual_conflicts_indep',
+	    GROUP => 'Core options',
 	    DEFAULT => [],
-	    HELP => 'Additional per-build dependencies.  Do not set by hand.'
+	    HELP => 'Additional per-build dependencies.',
+	    CLI_OPTIONS => ['--add-conflicts-indep']
 	},
 	'CROSSBUILD_CORE_DEPENDS'				=> {
 	    TYPE => 'HASH:ARRAY:STRING',
 	    VARNAME => 'crossbuild_core_depends',
 	    GROUP => 'Multiarch support (transitional)',
-	    DEFAULT => { arm64 => ['crossbuild-essential-arm64'],
-			 armel => ['crossbuild-essential-armel'],
-			 armhf => ['crossbuild-essential-armhf'],
-			 ia64 => ['crossbuild-essential-ia64'],
-			 mips => ['crossbuild-essential-mips'],
-			 mipsel => ['crossbuild-essential-mipsel'],
-			 powerpc => ['crossbuild-essential-powerpc'],
-			 sparc => ['crossbuild-essential-sparc']
-	    	       },
-	    HELP => 'Per-architecture dependencies required for cross-building.'
+	    DEFAULT => {},
+	    HELP => 'Per-architecture dependencies required for cross-building. By default, if a Debian architecture is not found as a key in this hash, the package crossbuild-essential-${hostarch}:native will be installed.',
+	    EXAMPLE => '
+$crossbuild_core_depends = {
+    nios2 => [\'crossbuild-essential-nios2:native\', \'special-package\'],
+    musl-linux-mips => [\'crossbuild-essential-musl-linux-mips:native\', \'super-special\'],
+}
+'
 	},	'BUILD_SOURCE'				=> {
 	    TYPE => 'BOOL',
 	    VARNAME => 'build_source',
 	    GROUP => 'Build options',
 	    DEFAULT => 0,
 	    CHECK => $validate_append_version,
-	    HELP => 'By default, do not build a source package (binary only build).  Set to 1 to force creation of a source package, but note that this is inappropriate for binary NMUs, where the option will always be disabled.'
+	    HELP => 'By default, do not build a source package (binary only build).  Set to 1 to force creation of a source package, but note that this is inappropriate for binary NMUs, where the option will always be disabled.',
+	    CLI_OPTIONS => ['-s', '--source', '--no-source']
 	},
 	'ARCHIVE'				=> {
 	    TYPE => 'STRING',
 	    VARNAME => 'archive',
 	    GROUP => 'Core options',
 	    DEFAULT => undef,
-	    HELP => 'Archive being built.  Only set in build log.  This might be useful for derivative distributions.'
+	    HELP => 'Archive being built.  Only set in build log.  This might be useful for derivative distributions.',
+	    CLI_OPTIONS => ['--archive']
 	},
 	'BIN_NMU'				=> {
 	    TYPE => 'STRING',
-	    GROUP => '__INTERNAL',
+	    VARNAME => 'bin_nmu',
+	    GROUP => 'Build options',
 	    DEFAULT => undef,
 	    CHECK => $validate_append_version,
-	    HELP => 'Binary NMU changelog entry.  Do not set by hand.'
+	    HELP => 'Binary NMU changelog entry.',
+	    CLI_OPTIONS => ['--make-binNMU']
 	},
 	'BIN_NMU_VERSION'			=> {
 	    TYPE => 'STRING',
-	    GROUP => '__INTERNAL',
+	    VARNAME => 'bin_nmu_version',
+	    GROUP => 'Build options',
 	    DEFAULT => undef,
-	    HELP => 'Binary NMU version number.  Do not set by hand.'
+	    HELP => 'Binary NMU version number.',
+	    CLI_OPTIONS => ['--binNMU', '--make-binNMU']
 	},
 	'APPEND_TO_VERSION'			=> {
 	    TYPE => 'STRING',
@@ -848,14 +963,16 @@ sub setup ($) {
 	    GROUP => 'Build options',
 	    DEFAULT => undef,
 	    CHECK => $validate_append_version,
-	    HELP => 'Suffix to append to version number.  May be useful for derivative distributions.'
+	    HELP => 'Suffix to append to version number.  May be useful for derivative distributions.',
+	    CLI_OPTIONS => ['--append-to-version']
 	},
 	'GCC_SNAPSHOT'				=> {
 	    TYPE => 'BOOL',
 	    VARNAME => 'gcc_snapshot',
 	    GROUP => 'Build options',
 	    DEFAULT => 0,
-	    HELP => 'Build using current GCC snapshot?'
+	    HELP => 'Build using current GCC snapshot?',
+	    CLI_OPTIONS => ['--use-snapshot']
 	},
 	'JOB_FILE'				=> {
 	    TYPE => 'STRING',
@@ -881,11 +998,28 @@ sub setup ($) {
 
 		die '$key: Invalid build-dependency resolver \'' .
 		    $conf->get($key) .
-		    "'\nValid algorithms are 'apt', 'aptitude' and 'xapt'\n"
+		    "'\nValid algorithms are 'apt', 'aptitude', 'aspcud' and 'xapt'\n"
 		    if !isin($conf->get($key),
-			     qw(apt aptitude xapt));
+			     qw(apt aptitude aspcud xapt));
 	    },
-	    HELP => 'Build dependency resolver.  The \'apt\' resolver is currently the default, and recommended for most users.  This resolver uses apt-get to resolve dependencies.  Alternative resolvers are \'apt\' and \'aptitude\', which use a built-in resolver module and aptitude to resolve build dependencies, respectively.  The aptitude resolver is similar to apt, but is useful in more complex situations, such as where multiple distributions are required, for example when building from experimental, where packages are needed from both unstable and experimental, but defaulting to unstable.'
+	    HELP => 'Build dependency resolver.  The \'apt\' resolver is currently the default, and recommended for most users.  This resolver uses apt-get to resolve dependencies.  Alternative resolvers are \'apt\', \'aptitude\' and \'aspcud\'. The \'apt\' resolver uses a built-in resolver module while the \'aptitude\' resolver uses aptitude to resolve build dependencies.  The aptitude resolver is similar to apt, but is useful in more complex situations, such as where multiple distributions are required, for example when building from experimental, where packages are needed from both unstable and experimental, but defaulting to unstable. If the dependency situation is too complex for either apt or aptitude to solve it, you can use the \'aspcud\' resolver which (in contrast to apt and aptitude) is a real solver (in the math sense) and will thus always find a solution if a solution exists.',
+	    CLI_OPTIONS => ['--build-dep-resolver']
+	},
+	'ASPCUD_CRITERIA'			=> {
+	    TYPE => 'STRING',
+	    VARNAME => 'aspcud_criteria',
+	    GROUP => 'Dependency resolution',
+	    DEFAULT => '-removed,-changed,-new',
+	    HELP => 'Optimization criteria in extended MISC 2012 syntax passed to aspcud through apt-cudf.  Optimization criteria are separated by commas, sorted by decreasing order of priority and are prefixed with a polarity (+ to maximize and - to minimize).  The default criteria is \'-removed,-changed,-new\' which first minimizes the number of removed packages, then the number of changed packages (up or downgrades) and then the number of new packages. A common task is to minimize the number of packages from experimental.  To do this you can add a criteria like \'-count(solution,APT-Release:=/a=experimental/)\' to the default criteria.  This will then minimize the number of packages in the solution which contain the string \'a=experimental\' in the \'APT-Release\' field of the EDSP output created by apt. See the apt-cudf man page help on the --criteria option for more information.',
+	    CLI_OPTIONS => ['--aspcud-criteria']
+	},
+	'CLEAN_SOURCE'				=> {
+	    TYPE => 'BOOL',
+	    VARNAME => 'clean_source',
+	    GROUP => 'Build options',
+	    DEFAULT => 1,
+	    HELP => 'When running sbuild from within an unpacked source tree, run the \'clean\' target before generating the source package. This might require some of the build dependencies necessary for running the \'clean\' target to be installed on the host machine. Only disable if you start from a clean checkout and you know what you are doing.',
+	    CLI_OPTIONS => ['--clean-source', '--no-clean-source']
 	},
 	'LINTIAN'				=> {
 	    TYPE => 'STRING',
@@ -913,14 +1047,16 @@ sub setup ($) {
 		$conf->check('LINTIAN');
 	    },
 	    DEFAULT => 0,
-	    HELP => 'Run lintian?'
+	    HELP => 'Run lintian?',
+	    CLI_OPTIONS => ['--run-lintian', '--no-run-lintian']
 	},
 	'LINTIAN_OPTIONS'			=> {
 	    TYPE => 'ARRAY:STRING',
 	    VARNAME => 'lintian_opts',
 	    GROUP => 'Build validation',
 	    DEFAULT => [],
-	    HELP => 'Options to pass to lintian.  Each option is a separate arrayref element.  For example, [\'-i\', \'-v\'] to add -i and -v.'
+	    HELP => 'Options to pass to lintian.  Each option is a separate arrayref element.  For example, [\'-i\', \'-v\'] to add -i and -v.',
+	    CLI_OPTIONS => ['--lintian-opt', '--lintian-opts']
 	},
 	'PIUPARTS'				=> {
 	    TYPE => 'STRING',
@@ -937,7 +1073,8 @@ sub setup ($) {
 		}
 	    },
 	    DEFAULT => 'piuparts',
-	    HELP => 'Path to piuparts binary'
+	    HELP => 'Path to piuparts binary',
+	    CLI_OPTIONS => ['--piuparts-opt', '--piuparts-opts']
 	},
 	'RUN_PIUPARTS'				=> {
 	    TYPE => 'BOOL',
@@ -948,13 +1085,41 @@ sub setup ($) {
 		$conf->check('PIUPARTS');
 	    },
 	    DEFAULT => 0,
-	    HELP => 'Run piuparts'
+	    HELP => 'Run piuparts',
+	    CLI_OPTIONS => ['--run-piuparts', '--no-run-piuparts']
 	},
 	'PIUPARTS_OPTIONS'			=> {
 	    TYPE => 'ARRAY:STRING',
 	    VARNAME => 'piuparts_opts',
 	    GROUP => 'Build validation',
 	    DEFAULT => [],
+	    GET => sub {
+		my $conf = shift;
+		my $entry = shift;
+
+		my $retval = $conf->_get($entry->{'NAME'});
+
+		my $dist = $conf->get('DISTRIBUTION');
+		my $hostarch = $conf->get('HOST_ARCH');
+		my %percent = (
+		    '%' => '%',
+		    'a' => $hostarch, 'SBUILD_HOST_ARCH' => $hostarch,
+		    'r' => $dist, 'SBUILD_DISTRIBUTION' => $dist,
+		);
+
+		my $keyword_pat = join("|",
+		    sort {length $b <=> length $a || $a cmp $b} keys %percent);
+		foreach (@{$retval}) {
+		    s{
+			# Match a percent followed by a valid keyword
+			\%($keyword_pat)
+		    }{
+			# Substitute with the appropriate value only if it's defined
+			$percent{$1} || $&
+		    }msxge;
+		}
+		return $retval;
+	    },
 	    HELP => 'Options to pass to piuparts.  Each option is a separate arrayref element.  For example, [\'-b\', \'<chroot_tarball>\'] to add -b and <chroot_tarball>.'
 	},
 	'PIUPARTS_ROOT_ARGS'			=> {
@@ -962,15 +1127,115 @@ sub setup ($) {
 	    VARNAME => 'piuparts_root_args',
 	    GROUP => 'Build validation',
 	    DEFAULT => [],
-	    HELP => 'Preceding arguments to launch piuparts as root. If no arguments are specified, piuparts will be launched via sudo.'
+	    HELP => 'Preceding arguments to launch piuparts as root. With the default value (the empty array) "sudo --" will be used as a prefix. If the first element in the array is the empty string, no prefixing will be done. If the value is a scalar, it will be prefixed by that string. If the scalar is an empty string, no prefixing will be done.',
+	    EXAMPLE =>
+'# prefix with "sudo --":
+$piuparts_root_args = [];
+$piuparts_root_args = [\'sudo\', \'--\'];
+# prefix with "env":
+$piuparts_root_args = [\'env\'];
+$piuparts_root_args = \'env\';
+# prefix with nothing:
+$piuparts_root_args = \'\';
+$piuparts_root_args = [\'\'];
+$piuparts_root_args = [\'\', \'whatever\'];
+',
+	    CLI_OPTIONS => ['--piuparts-root-arg', '--piuparts-root-args']
+	},
+	'AUTOPKGTEST'				=> {
+	    TYPE => 'STRING',
+	    VARNAME => 'autopkgtest',
+	    GROUP => 'Build validation',
+	    CHECK => sub {
+		my $conf = shift;
+		my $entry = shift;
+		my $key = $entry->{'NAME'};
+
+		# Only validate if needed.
+		if ($conf->get('RUN_AUTOPKGTEST')) {
+		    $validate_program->($conf, $entry);
+		}
+	    },
+	    DEFAULT => 'autopkgtest',
+	    HELP => 'Path to autopkgtest binary',
+	    CLI_OPTIONS => ['--autopkgtest-opt', '--autopkgtest-opts']
+	},
+	'RUN_AUTOPKGTEST'				=> {
+	    TYPE => 'BOOL',
+	    VARNAME => 'run_autopkgtest',
+	    GROUP => 'Build validation',
+	    CHECK => sub {
+		my $conf = shift;
+		$conf->check('AUTOPKGTEST');
+	    },
+	    DEFAULT => 0,
+	    HELP => 'Run autopkgtest',
+	    CLI_OPTIONS => ['--run-autopkgtest', '--no-run-autopkgtest']
+	},
+	'AUTOPKGTEST_OPTIONS'			=> {
+	    TYPE => 'ARRAY:STRING',
+	    VARNAME => 'autopkgtest_opts',
+	    GROUP => 'Build validation',
+	    DEFAULT => [],
+	    GET => sub {
+		my $conf = shift;
+		my $entry = shift;
+
+		my $retval = $conf->_get($entry->{'NAME'});
+
+		my $dist = $conf->get('DISTRIBUTION');
+		my $hostarch = $conf->get('HOST_ARCH');
+		my %percent = (
+		    '%' => '%',
+		    'a' => $hostarch, 'SBUILD_HOST_ARCH' => $hostarch,
+		    'r' => $dist, 'SBUILD_DISTRIBUTION' => $dist,
+		);
+
+		my $keyword_pat = join("|",
+		    sort {length $b <=> length $a || $a cmp $b} keys %percent);
+		foreach (@{$retval}) {
+		    s{
+			# Match a percent followed by a valid keyword
+			\%($keyword_pat)
+		    }{
+			# Substitute with the appropriate value only if it's defined
+			$percent{$1} || $&
+		    }msxge;
+		}
+		return $retval;
+	    },
+	    HELP => 'Options to pass to autopkgtest.  Each option is a separate arrayref element.  For example, [\'-b\', \'<chroot_tarball>\'] to add -b and <chroot_tarball>.'
+	},
+	'AUTOPKGTEST_ROOT_ARGS'			=> {
+	    TYPE => 'ARRAY:STRING',
+	    VARNAME => 'autopkgtest_root_args',
+	    GROUP => 'Build validation',
+	    DEFAULT => [],
+	    HELP => 'Preceding arguments to launch autopkgtest as root. With the default value (the empty array) "sudo --" will be used as a prefix. If the first element in the array is the empty string, no prefixing will be done. If the value is a scalar, it will be prefixed by that string. If the scalar is an empty string, no prefixing will be done.',
+	    EXAMPLE =>
+'# prefix with "sudo --":
+$autopkgtest_root_args = [];
+$autopkgtest_root_args = [\'sudo\', \'--\'];
+# prefix with "env":
+$autopkgtest_root_args = [\'env\'];
+$autopkgtest_root_args = \'env\';
+# prefix with nothing:
+$autopkgtest_root_args = \'\';
+$autopkgtest_root_args = [\'\'];
+$autopkgtest_root_args = [\'\', \'whatever\'];
+',
+	    CLI_OPTIONS => ['--autopkgtest-root-arg', '--autopkgtest-root-args']
 	},
 	'EXTERNAL_COMMANDS'			=> {
-	    TYPE => 'HASH:ARRAY:ARRAY:STRING',
+	    TYPE => 'HASH:ARRAY:STRING',
 	    VARNAME => 'external_commands',
 	    GROUP => 'Chroot options',
 	    DEFAULT => {
 		"pre-build-commands" => [],
 		"chroot-setup-commands" => [],
+		"chroot-update-failed-commands" => [],
+		"build-deps-failed-commands" => [],
+		"build-failed-commands" => [],
 		"starting-build-commands" => [],
 		"finished-build-commands" => [],
 		"chroot-cleanup-commands" => [],
@@ -987,6 +1252,18 @@ sub setup ($) {
         [\'foo\', \'arg1\', \'arg2\'],
         [\'bar\', \'arg1\', \'arg2\', \'arg3\'],
     ],
+    "chroot-update-failed-commands" => [
+        [\'foo\', \'arg1\', \'arg2\'],
+        [\'bar\', \'arg1\', \'arg2\', \'arg3\'],
+    ],
+    "build-deps-failed-commands" => [
+        [\'foo\', \'arg1\', \'arg2\'],
+        [\'bar\', \'arg1\', \'arg2\', \'arg3\'],
+    ],
+    "build-failed-commands" => [
+        [\'foo\', \'arg1\', \'arg2\'],
+        [\'bar\', \'arg1\', \'arg2\', \'arg3\'],
+    ],
     "starting-build-commands" => [
         [\'foo\', \'arg1\', \'arg2\'],
         [\'bar\', \'arg1\', \'arg2\', \'arg3\'],
@@ -1003,21 +1280,24 @@ sub setup ($) {
         [\'foo\', \'arg1\', \'arg2\'],
         [\'bar\', \'arg1\', \'arg2\', \'arg3\'],
     ],
-};'
+};',
+	    CLI_OPTIONS => ['--setup-hook', '--pre-build-commands', '--chroot-setup-commands', '--chroot-update-failed-commands', '--build-deps-failed-commands', '--build-failed-commands', '--anything-failed-commands', '--starting-build-commands', '--finished-build-commands', '--chroot-cleanup-commands', '--post-build-commands']
 	},
 	'LOG_EXTERNAL_COMMAND_OUTPUT'		=> {
 	    TYPE => 'BOOL',
 	    VARNAME => 'log_external_command_output',
 	    GROUP => 'Chroot options',
 	    DEFAULT => 1,
-	    HELP => 'Log standard output of commands run by sbuild?'
+	    HELP => 'Log standard output of commands run by sbuild?',
+	    CLI_OPTIONS => ['--log-external-command-output']
 	},
 	'LOG_EXTERNAL_COMMAND_ERROR'		=> {
 	    TYPE => 'BOOL',
 	    VARNAME => 'log_external_command_error',
 	    GROUP => 'Chroot options',
 	    DEFAULT => 1,
-	    HELP => 'Log standard error of commands run by sbuild?'
+	    HELP => 'Log standard error of commands run by sbuild?',
+	    CLI_OPTIONS => ['--log-external-command-error']
 	},
 	'RESOLVE_ALTERNATIVES'				=> {
 	    TYPE => 'BOOL',
@@ -1039,7 +1319,8 @@ sub setup ($) {
 		return $retval;
 	    },
 	    EXAMPLE => '$resolve_alternatives = 0;',
-	    HELP => 'Should the dependency resolver use alternatives in Build-Depends, Build-Depends-Arch and Build-Depends-Indep?  By default, using \'apt\' resolver, only the first alternative will be used; all other alternatives will be removed.  When using the \'aptitude\' resolver, it will default to using all alternatives.  Note that this does not include architecture-specific alternatives, which are reduced to the build architecture prior to alternatives removal.  This should be left disabled when building for unstable; it may be useful when building for experimental or backports.  Set to undef to use the default, 1 to enable, or 0 to disable.'
+	    HELP => 'Should the dependency resolver use alternatives in Build-Depends, Build-Depends-Arch and Build-Depends-Indep?  By default, using \'apt\' resolver, only the first alternative will be used; all other alternatives will be removed.  When using the \'aptitude\' resolver, it will default to using all alternatives.  Note that this does not include architecture-specific alternatives, which are reduced to the build architecture prior to alternatives removal.  This should be left disabled when building for unstable; it may be useful when building for experimental or backports.  Set to undef to use the default, 1 to enable, or 0 to disable.',
+	    CLI_OPTIONS => ['--resolve-alternatives', '--no-resolve-alternatives']
 	},
 	'SBUILD_BUILD_DEPENDS_SECRET_KEY'		=> {
 	    TYPE => 'STRING',
@@ -1057,16 +1338,62 @@ sub setup ($) {
 	},
 	'EXTRA_PACKAGES'				=> {
 	    TYPE => 'ARRAY:STRING',
-	    GROUP => '__INTERNAL',
+	    VARNAME => 'extra_packages',
+	    GROUP => 'Dependency resolution',
 	    DEFAULT => [],
-	    HELP => 'Additional per-build packages available as build dependencies.  Do not set by hand.'
+	    HELP => 'Additional per-build packages available as build dependencies.',
+	    CLI_OPTIONS => ['--extra-package']
+	},
+	'EXTRA_REPOSITORY_KEYS'				=> {
+	    TYPE => 'ARRAY:STRING',
+	    VARNAME => 'extra_repository_keys',
+	    GROUP => 'Dependency resolution',
+	    DEFAULT => [],
+	    HELP => 'Additional per-build apt repository keys.',
+	    CLI_OPTIONS => ['--extra-repository-key']
 	},
 	'EXTRA_REPOSITORIES'				=> {
 	    TYPE => 'ARRAY:STRING',
-	    GROUP => '__INTERNAL',
+	    VARNAME => 'extra_repositories',
+	    GROUP => 'Dependency resolution',
 	    DEFAULT => [],
-	    HELP => 'Additional per-build apt repositories.  Do not set by hand.'
+	    HELP => 'Additional per-build apt repositories.',
+	    CLI_OPTIONS => ['--extra-repository']
 	},
+	'SOURCE_ONLY_CHANGES'				=> {
+	    TYPE => 'BOOL',
+	    VARNAME => 'source_only_changes',
+	    GROUP => 'Build options',
+	    DEFAULT => 0,
+	    HELP => 'Also produce a changes file suitable for a source-only upload.',
+	    CLI_OPTIONS => ['--source-only-changes']
+	},
+	'BD_UNINSTALLABLE_EXPLAINER'			=> {
+	    TYPE => 'STRING',
+	    VARNAME => 'bd_uninstallable_explainer',
+	    GROUP => 'Dependency resolution',
+	    CHECK => sub {
+		my $conf = shift;
+		my $entry = shift;
+		my $key = $entry->{'NAME'};
+
+		die "Bad bd-uninstallable explainer \'" . $conf->get('BD_UNINSTALLABLE_EXPLAINER') . "\'"
+		    if defined $conf->get('BD_UNINSTALLABLE_EXPLAINER')
+		    && !isin($conf->get('BD_UNINSTALLABLE_EXPLAINER'),
+			     ('apt', 'dose3', ''));
+	    },
+	    DEFAULT => 'dose3',
+	    HELP => 'Method to use for explaining build dependency installation failures. Possible value are "dose3" (default) and "apt". Set to the empty string or undef to disable running any explainer.',
+	    CLI_OPTIONS => ['--bd-uninstallable-explainer']
+	},
+	'PURGE_EXTRA_PACKAGES'				=> {
+	    TYPE => 'BOOL',
+	    VARNAME => 'purge_extra_packages',
+	    GROUP => 'Chroot options',
+	    DEFAULT => 0,
+	    HELP => 'Try to remove all additional packages that are not strictly required for the build right after build dependencies were installed. This currently works best with the aspcud resolver. The apt resolver will not make as much effort to remove all unneeded packages and will keep all providers of a virtual package and all packages from any dependency alternative that happen to be installed. The aptitude and xapt resolver do not implement this feature yet. The removed packages are not yet added again after the build finished. This can have undesirable side effects like lintian not working (because there is no apt to install its dependencies) or bare chroots becoming totally unusable after apt was removed from them. Thus, this option should only be used with throw-away chroots like schroot provides them where the original state is automatically restored after each build.',
+	    CLI_OPTIONS => ['--purge-extra-packages']
+	}
     );
 
     $conf->set_allowed_keys(\%sbuild_keys);
@@ -1087,6 +1414,8 @@ sub read ($) {
     my $files = ["$Sbuild::Sysconfig::paths{'SBUILD_CONF'}",
 		 "$HOME/.sbuildrc"];
 
+    push @{$files}, $ENV{'SBUILD_CONFIG'} if defined $ENV{'SBUILD_CONFIG'};
+
     # For compatibility only.  Non-scalars are deprecated.
     my $deprecated_init = <<END;
 my \%mailto;
